@@ -1,17 +1,14 @@
 use std::ops::{Add, AddAssign, Div, Mul, Sub};
 
-use peace_performance::{
-    parse::{
-        DifficultyPoint as RawDifficultyPoint, HitObject as RawHitObject,
-        HitObjectKind as RawHitObjectKind, PathType, Pos2 as RawPos2,
-        TimingPoint as RawTimingPoint,
-    },
-    Beatmap as RawBeatmap,
-};
 use pyo3::{
-    prelude::{pyclass, pymethods, pyproto},
+    prelude::{pyclass, pymethods},
     types::PyDict,
-    PyObjectProtocol, PyResult, Python,
+    PyResult, Python,
+};
+use rosu_pp::{
+    beatmap::{DifficultyPoint as RawDifficultyPoint, TimingPoint as RawTimingPoint},
+    parse::{HitObject as RawHitObject, HitObjectKind as RawHitObjectKind, Pos2 as RawPos2},
+    Beatmap as RawBeatmap,
 };
 
 use crate::methods::common::osu_mode_str;
@@ -19,9 +16,8 @@ use crate::methods::common::osu_mode_str;
 #[pyclass]
 #[derive(Clone, Default, Debug)]
 pub struct Beatmap(pub RawBeatmap);
-crate::pyo3_py_protocol!(Beatmap);
-crate::pyo3_py_methods!(
-    Beatmap, {
+crate::py_impl!(
+    for Beatmap, @attrs {
         version: u8,
         n_circles: u32,
         n_sliders: u32,
@@ -30,9 +26,8 @@ crate::pyo3_py_methods!(
         od: f32,
         cs: f32,
         hp: f32,
-        sv: f32,
-        tick_rate: f32
-    }, impl {
+        tick_rate: f64
+    }; @getters {}; @methods {
         #[getter]
         pub fn mode(&self) -> u8 {
             self.0.mode as u8
@@ -43,6 +38,11 @@ crate::pyo3_py_methods!(
             osu_mode_str(&self.0.mode)
         }
 
+        #[getter]
+        pub fn sv(&self) -> f64 {
+            self.0.slider_mult
+        }
+
         #[getter]
         pub fn hit_objects(&self) -> Option<Vec<HitObject>> {
             Some(self.0
@@ -111,7 +111,7 @@ crate::pyo3_py_methods!(
                 self.0.od,
                 self.0.cs,
                 self.0.hp,
-                self.0.sv,
+                self.0.slider_mult,
                 self.0.tick_rate,
                 self.stack_leniency()
             )
@@ -121,22 +121,18 @@ crate::pyo3_py_methods!(
         #[inline(always)]
         pub fn as_dict<'a>(&self, py: Python<'a>) -> PyResult<&'a PyDict> {
             let d = crate::pyo3_py_dict!(py, self.0; {
-                version, n_circles, n_sliders, n_spinners, ar, od, cs, hp, sv, tick_rate
-            }; fn self {mode_str, stack_leniency});
+                version, n_circles, n_sliders, n_spinners, ar, od, cs, hp, tick_rate
+            }; fn self {mode_str, stack_leniency, sv});
             d.set_item("mode", self.0.mode as u8)?;
             Ok(d)
         }
     }
 );
 
-#[cfg(any(feature = "osu", feature = "fruits"))]
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct DifficultyPoint(pub RawDifficultyPoint);
-#[cfg(any(feature = "osu", feature = "fruits"))]
-crate::pyo3_py_protocol!(DifficultyPoint);
-#[cfg(any(feature = "osu", feature = "fruits"))]
-crate::pyo3_py_methods!(DifficultyPoint, {time: f32, speed_multiplier: f32}, impl {
+crate::py_impl!(for DifficultyPoint, @attrs {time: f64, speed_multiplier: f64}; @getters {}; @methods {
     #[getter]
     #[inline(always)]
     pub fn as_string(&self) -> String {
@@ -154,8 +150,7 @@ crate::pyo3_py_methods!(DifficultyPoint, {time: f32, speed_multiplier: f32}, imp
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct TimingPoint(pub RawTimingPoint);
-crate::pyo3_py_protocol!(TimingPoint);
-crate::pyo3_py_methods!(TimingPoint, {time: f32, beat_len: f32}, impl {
+crate::py_impl!(for TimingPoint, @attrs {time: f64, beat_len: f64}; @getters {}; @methods {
     #[getter]
     #[inline(always)]
     pub fn as_string(&self) -> String {
@@ -173,11 +168,10 @@ crate::pyo3_py_methods!(TimingPoint, {time: f32, beat_len: f32}, impl {
 #[pyclass]
 #[derive(Clone, Default, Debug)]
 pub struct Pos2(pub RawPos2);
-crate::pyo3_py_protocol!(Pos2);
-crate::pyo3_py_methods!(Pos2, {x: f32, y: f32}; fn {
+crate::py_impl!(for Pos2, @attrs {x: f32, y: f32}; @getters {
     length_squared: f32,
     length: f32
-}; impl {
+}; @methods {
     pub fn dot(&self, other: Self) -> f32 {
         self.0.dot(other.0)
     }
@@ -236,7 +230,7 @@ pub struct HitObjectKind {
     #[pyo3(get)]
     pub kind: String,
     #[pyo3(get)]
-    pub pixel_len: Option<f32>,
+    pub pixel_len: Option<f64>,
     #[pyo3(get)]
     pub repeats: Option<usize>,
     #[pyo3(get)]
@@ -244,10 +238,9 @@ pub struct HitObjectKind {
     #[pyo3(get)]
     pub path_type: Option<String>,
     #[pyo3(get)]
-    pub end_time: Option<f32>,
+    pub end_time: Option<f64>,
 }
-crate::pyo3_py_protocol!(HitObjectKind);
-crate::pyo3_py_methods!(HitObjectKind, impl {
+crate::py_impl!(for HitObjectKind, @attrs {}; @getters {}; @methods {
     #[getter]
     #[inline(always)]
     pub fn as_string(&self) -> String {
@@ -276,13 +269,12 @@ crate::pyo3_py_methods!(HitObjectKind, impl {
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct HitObject(pub RawHitObject);
-crate::pyo3_py_protocol!(HitObject);
-crate::pyo3_py_methods!(HitObject, {start_time: f32, sound: u8}; fn {
-    end_time: f32,
+crate::py_impl!(for HitObject, @attrs {start_time: f64}; @getters {
+    end_time: f64,
     is_circle: bool,
     is_slider: bool,
     is_spinner: bool
-}; impl {
+}; @methods {
     #[getter]
     pub fn pos(&self) -> Pos2 {
         Pos2(self.0.pos)
@@ -324,7 +316,7 @@ crate::pyo3_py_methods!(HitObject, {start_time: f32, sound: u8}; fn {
                 feature = "fruits",
                 all(feature = "osu", not(feature = "no_sliders_no_leniency"))
             )))]
-            RawHitObjectKind::Slider { pixel_len, repeats } => HitObjectKind {
+            RawHitObjectKind::Slider { pixel_len, repeats, .. } => HitObjectKind {
                 kind: "slider".into(),
                 pixel_len: Some(*pixel_len),
                 repeats: Some(*repeats),
@@ -346,9 +338,8 @@ crate::pyo3_py_methods!(HitObject, {start_time: f32, sound: u8}; fn {
     #[getter]
     #[inline(always)]
     pub fn as_string(&self) -> String {
-        format!("start_time: {}, sound: {}, end_time: {}, kind: {}, pos: ({}, {})",
+        format!("start_time: {}, end_time: {}, kind: {}, pos: ({}, {})",
             self.0.start_time,
-            self.0.sound,
             self.end_time(),
             self.kind_str(),
             self.0.pos.x,
@@ -359,7 +350,7 @@ crate::pyo3_py_methods!(HitObject, {start_time: f32, sound: u8}; fn {
     #[getter]
     #[inline(always)]
     pub fn as_dict<'a>(&self, py: Python<'a>) -> PyResult<&'a PyDict> {
-        let d = crate::pyo3_py_dict!(py, self.0; {start_time, sound}; fn self {
+        let d = crate::pyo3_py_dict!(py, self.0; {start_time}; fn self {
             end_time
         });
         d.set_item("kind", self.kind().as_dict(py)?)?;
