@@ -1,12 +1,16 @@
-use peace_performance::PpResult;
 use pyo3::{
-    prelude::{pyclass, pymethods, pyproto},
+    prelude::{pyclass, pymethods},
     types::PyDict,
-    PyAny, PyCell, PyObjectProtocol, PyResult, Python,
+    PyAny, PyCell, PyResult, Python,
 };
+use rosu_pp::PerformanceAttributes;
 
 use super::CalcResult;
-use crate::{methods::pp, objects::Beatmap, set_calculator};
+use crate::{
+    methods::pp::{self, PpRaw, PpResult},
+    objects::Beatmap,
+    set_calculator,
+};
 
 macro_rules! create_py_methods {
     ($for:ident, fn get_set_del() {$($attr:ident: $type:ty),*}, impl {$($others:tt)*}) => {
@@ -58,7 +62,7 @@ pub struct Calculator {
     #[pyo3(get, set)]
     pub katu: Option<usize>,
     #[pyo3(get, set)]
-    pub acc: Option<f32>,
+    pub acc: Option<f64>,
     #[pyo3(get, set)]
     pub passed_obj: Option<usize>,
     #[pyo3(get, set)]
@@ -68,7 +72,6 @@ pub struct Calculator {
     #[pyo3(get, set)]
     pub score: Option<u32>,
 }
-crate::pyo3_py_protocol!(Calculator);
 create_py_methods!(
     Calculator,
     fn get_set_del() {
@@ -78,7 +81,7 @@ create_py_methods!(
         n100: Option<usize>,
         n300: Option<usize>,
         katu: Option<usize>,
-        acc: Option<f32>,
+        acc: Option<f64>,
         passed_obj: Option<usize>,
         combo: Option<usize>,
         miss: Option<usize>,
@@ -190,27 +193,45 @@ impl Calculator {
     #[inline(always)]
     #[cfg_attr(feature = "rust_logger", timed::timed(duration(printer = "trace!")))]
     pub fn calc(&self, beatmap: &Beatmap) -> PpResult {
-        let c = pp::mode_any_pp(self.mode.unwrap_or(4), &beatmap.0);
-        let c = set_calculator!(self.mods, c);
-        // Irrelevant for osu!mania
-        let c = set_calculator!(self.combo, c);
-        // Irrelevant for osu!mania and osu!taiko
-        let c = set_calculator!(self.n50, c);
-        // Irrelevant for osu!mania
-        let c = set_calculator!(self.n100, c);
-        // Irrelevant for osu!mania
-        let c = set_calculator!(self.n300, c);
-        // Only relevant for osu!ctb
-        let c = set_calculator!(self.katu, n_katu, c);
-        // Irrelevant for osu!mania
-        let c = set_calculator!(self.miss, misses, c);
-        let c = set_calculator!(self.passed_obj, passed_objects, c);
-        // Only relevant for osu!mania
-        let mut c = set_calculator!(self.score, c);
-        // Irrelevant for osu!mania
-        if let Some(acc) = self.acc {
-            c.set_accuracy(acc)
+        let c = pp::mode_any_pp(self.mode, &beatmap.0);
+
+        let c = set_calculator!(c, self.mods, self.combo, self.n50, self.n100, self.n300, {
+            self.katu: n_katu,
+            self.miss: misses,
+            self.passed_obj: passed_objects
+        });
+        let c = set_calculator!(c, self.score, { self.acc: accuracy });
+
+        let attributes = c.calculate();
+        let (mode, raw) = match &attributes {
+            PerformanceAttributes::Osu(attr) => (
+                0,
+                PpRaw::new(
+                    Some(attr.pp_aim),
+                    Some(attr.pp_speed),
+                    None,
+                    Some(attr.pp_acc),
+                    attr.pp,
+                ),
+            ),
+            PerformanceAttributes::Taiko(attr) => (
+                1,
+                PpRaw::new(None, None, Some(attr.pp_strain), Some(attr.pp_acc), attr.pp),
+            ),
+            PerformanceAttributes::Catch(attr) => {
+                (2, PpRaw::new(Some(attr.pp), None, None, None, attr.pp))
+            }
+            PerformanceAttributes::Mania(attr) => (
+                3,
+                PpRaw::new(None, None, Some(attr.pp_strain), Some(attr.pp_acc), attr.pp),
+            ),
         };
-        pp::calc_with_any_pp(&mut c)
+        PpResult {
+            mode,
+            mods: self.mods.unwrap_or(0),
+            pp: attributes.pp(),
+            raw,
+            attributes,
+        }
     }
 }
